﻿using Mysqlx.Resultset;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace MontoyaTech.MySqlPlus
{
    /// <summary>
    /// A set of extensions to help work with MySqlColumns.
    /// </summary>
    public static class MySqlColumnExtensions
    {
        /// <summary>
        /// Gets the FieldInfo and MySqlColumn for an Id column on a given row and returns
        /// whether or not we found it.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="row"></param>
        /// <param name="idField"></param>
        /// <param name="idColumn"></param>
        public static bool GetMySqlId<T>(this T row, out FieldInfo idField, out MySqlColumn idColumn)
        {
            var type = typeof(T);

            return GetMySqlId(type, out idField, out idColumn);
        }

        /// <summary>
        /// Gets the FieldInfo and MySqlColumn for an Id column on a given row type and returns
        /// whether or not we found it.
        /// </summary>
        /// <param name="type"></param>
        /// <param name="idField"></param>
        /// <param name="idColumn"></param>
        /// <returns></returns>
        public static bool GetMySqlId(this Type type, out FieldInfo idField, out MySqlColumn idColumn)
        {
            idField = null;

            idColumn = null;

            var fields = type.GetFields();

            if (fields == null || fields.Length == 0)
                return false;

            for (int i = 0; i < fields.Length; i++)
            {
                var column = fields[i].GetCustomAttribute<MySqlColumn>();

                if (column != null && column.Id)
                {
                    idField = fields[i];
                    idColumn = column;

                    return true;
                }
            }

            return false;
        }
    }
}
